function [rowClus, colClus, z, U, V, obj] = proxi3_1for4cluster(z1, M, sz, sv,vargin) 
% Exact same u among different views is not assumed. Individual u for each
% view. Views are connected by introducing an extra variable: vect z, 
% such that u_i \dot z as the left vector in rank one approximation.

% min (1/2) \sum ||M_i - (z \dot u_i)v_i^T||_F^2
% s.t. ||z||_0 <= sz, ||v_i||_0 <= sv(i)

debug = true;

maxIter = 1e05;
threshold = 1e-03;

n = size(M{1}, 1);
nView = length(M);
d = zeros(nView, 1);
z = ones(n,1); % initialize z with all ones
z=z1;
U = ones(n, nView); % initialize u with all ones
V = cell(nView, 1);

gamma_z = 1.2;
gamma_u = 1.2;
gamma_v = 1.2;
for iView = 1:nView
    d(iView) = size(M{iView}, 2);
    V{iView} = ones(d(iView), 1); % initialize v with all ones
end

if debug
    fprintf('Initial objective: %2.2e\n', objective(M, z, U, V));
end

% first iteration
% starts from first view
% V{1} = update_v(M{1}, z, U(:, 1), V{1}, gamma_v, sv(1)); % update v
V{1}(1:end) = 0;
if nargin<5
    V{1}(1) = 1;
elseif vargin(1)>length(V{1})
    error('Inital feature exceeds the total number of features.');
else
    V{1}(vargin(1)) = 1;
end
U(:, 1) = update_u(M{1}, z, U(:, 1), V{1}, gamma_u); % update u
% U(:, 1) = map(U(:, 1), sz);
% update z
tmp = map(U(:, 1), sz);
z(tmp == 0) = 0;
% for the rest view
if nView > 1
    for iView = 2:nView
        V{iView} = update_v(M{iView}, z, U(:, iView), V{iView}, gamma_v, sv(iView)); % update v
        U(:, iView) = update_u(M{iView}, z, U(:, iView), V{iView}, gamma_u); % update u
    end
    % update z
    z = update_z(M, z, U, V, gamma_z, sz);
end

obj = objective(M, z, U, V);
if debug
    fprintf('Iter 1 -- objective: %2.2e\n', obj);
end

for iIter = 1:maxIter
    pre_obj = obj;
    
    pre_z = z;
%     pre_U = U;
%     pre_V = V;
    % given z, update U, V
    for iView = 1:nView
        V{iView} = update_v(M{iView}, z, U(:, iView), V{iView}, gamma_v, sv(iView)); % update v
        U(:, iView) = update_u(M{iView}, z, U(:, iView), V{iView}, gamma_u); % update u 
    end

    % given U, V, update z    
    z = update_z(M, z, U, V, gamma_z, sz);
    
    % calculate the objective
    obj = objective(M, z, U, V);
    
    if debug
        fprintf('Iter %d -- objective: %2.2e, change in obj: %2.2e, in z: %2.2e\n', ...
            iIter, obj, pre_obj - obj, norm(pre_z - z));
    end
    
    if norm(pre_z - z) < threshold
        break;
    end
end

if iIter == maxIter
    warning('MVBC10 does not converge, try to increase the maximum iteration limit');
end
% generate clusters
rowClus = zeros(n, 1); % row clusters
rowClus(z ~= 0) = 1;
colClus = cell(nView, 1); % column clusters
for i = 1:nView
    colClus{i} = zeros(d(i), 1);
    colClus{i}(V{i} ~= 0) = 1;
end

end














